@@ -20,13 +20,29 @@ var NOT_EDITABLE = ['from', 'until', 'width', 'height', 'target', 'uniq', '_uniq
 var editor = null;
 
 var cookieProvider = new Ext.state.CookieProvider({
-  path: "/dashboard"
+  path: document.body.dataset.baseUrl + "dashboard"
 });
 
 var NAV_BAR_REGION = cookieProvider.get('navbar-region') || 'north';
 
 var CONFIRM_REMOVE_ALL = cookieProvider.get('confirm-remove-all') != 'false';
 
+var currently_setting_hash = false;
+
+function changeHash(hash){
+    currently_setting_hash = true;
+    window.location.hash = hash;
+}
+
+if ("onhashchange" in window) // does the browser support the hashchange event?
+  window.onhashchange = function () {
+    if (currently_setting_hash){
+      currently_setting_hash = false;
+      return;
+    }
+    location.reload();
+  }
+
 /* Nav Bar configuration */
 var navBarNorthConfig = {
   region: 'north',
@@ -34,6 +50,7 @@ var navBarNorthConfig = {
   layoutConfig: { align: 'stretch' },
   collapsible: true,
   collapseMode: 'mini',
+  collapsed: false,
   split: true,
   title: "untitled",
   height: 350,
@@ -69,7 +86,7 @@ var ContextFieldValueRecord = Ext.data.Record.create([
 ]);
 
 var contextFieldStore = new Ext.data.JsonStore({
-  url: '/metrics/find/',
+  url: document.body.dataset.baseUrl + 'metrics/find/',
   root: 'metrics',
   idProperty: 'name',
   fields: ContextFieldValueRecord,
@@ -82,7 +99,8 @@ var GraphRecord = new Ext.data.Record.create([
   {name: 'params', type: 'auto'},
   {name: 'url'},
   {name: 'width', type: 'auto'},
-  {name: 'height', type: 'auto'}
+  {name: 'height', type: 'auto'},
+  {name: 'loading'},
 ]);
 
 var graphStore;
@@ -115,6 +133,18 @@ if (sessionDefaultParamsJson && sessionDefaultParamsJson.length > 0) {
   defaultGraphParams = Ext.apply({}, originalDefaultGraphParams);
 }
 
+function isLoggedIn() {
+  return userName != null;
+}
+
+function hasPermission(permission) {
+  for (i in permissions) {
+    if (permissions[i] === permission) {
+      return true;
+    }
+  }
+  return false;
+}
 
 function initDashboard () {
 
@@ -287,7 +317,7 @@ function initDashboard () {
       }),
       store: new Ext.data.JsonStore({
         method: 'GET',
-        url: '/metrics/find/',
+        url: document.body.dataset.baseUrl + 'metrics/find/',
         autoLoad: true,
         baseParams: {
           query: '',
@@ -350,7 +380,7 @@ function initDashboard () {
     '<tpl for=".">',
       '<div class="graph-container">',
         '<div class="graph-overlay">',
-          '<img class="graph-img" src="{url}" width="{width}" height="{height}">',
+          '<img class="graph-img{loading}" src="{url}" width="{width}" height="{height}">',
           '<div class="overlay-close-button" onclick="javascript: graphStore.removeAt(\'{index}\'); updateGraphRecords(); justClosedGraph = true;">X</div>',
         '</div>',
       '</div>',
@@ -510,6 +540,7 @@ function initDashboard () {
     text: getTimeText()
   };
 
+  // Note that some of these items are changed in postLoginMenuAdjust() after login/logout
   var dashboardMenu = {
     text: 'Dashboard',
     menu: {
@@ -526,22 +557,41 @@ function initDashboard () {
         }, {
           text: "Finder",
           handler: showDashboardFinder
+        }, {
+          text: "Template Finder",
+          handler: showTemplateFinder
+        }, {
+          text: "Save As Template",
+          handler: saveTemplate,
+          disabled: !hasPermission('change')
         }, {
           id: 'dashboard-save-button',
           text: "Save",
           handler: function (item, e) {
                      sendSaveRequest(dashboardName);
                    },
-          disabled: (dashboardName == null) ? true : false
+          disabled: dashboardName == null || !hasPermission('change')
         }, {
+          id: "dashboard-save-as-button",
           text: "Save As",
-          handler: saveDashboard
+          handler: saveDashboard,
+          disabled: !hasPermission('change')
         }, {
           text: "Configure UI",
           handler: configureUI
         }, {
           text: "Edit Dashboard",
           handler: editDashboard
+        }, {
+          id: "dashboard-login-button",
+          text: getLoginMenuItemText(),
+          handler: function (item, e) {
+                     if (isLoggedIn()) {
+                       logout();
+                     } else {
+                       showLoginForm();
+                     }
+                   }
         }
       ]
     }
@@ -562,6 +612,9 @@ function initDashboard () {
               },
               { text: "From Saved Graph",
                 handler: newFromSavedGraph
+              },
+              { text: "From Metric",
+                handler: newFromMetric
               }
             ]
           }
@@ -719,12 +772,18 @@ function initDashboard () {
   // Load initial dashboard state if it was passed in
   if (initialState) {
     applyState(initialState);
-    navBar.collapse();
+    navBar.collapse(false);
   }
 
   if(window.location.hash != '')
   {
-    sendLoadRequest(window.location.hash.substr(1));
+    if (window.location.hash.indexOf("/") != -1) {
+      var name_val = window.location.hash.substr(1).split('#');
+      sendLoadTemplateRequest(name_val[0],name_val[1]);
+    } else {
+      sendLoadRequest(window.location.hash.substr(1));
+    }
+    navBar.collapse(false);
   }
 
   if (initialError) {
@@ -738,7 +797,7 @@ function showHelp() {
     modal: true,
     width: 550,
     height: 300,
-    autoLoad: "/dashboard/help/"
+    autoLoad: document.body.dataset.baseUrl + "dashboard/help/"
   });
   win.show();
 }
@@ -870,7 +929,7 @@ function metricTreeSelectorShow(pattern) {
   }
 
   var loader = new Ext.tree.TreeLoader({
-    url: '/metrics/find/',
+    url: document.body.dataset.baseUrl + 'metrics/find/',
     requestMethod: 'GET',
     listeners: {beforeload: setParams}
   });
@@ -898,7 +957,12 @@ function metricTextSelectorShow(pattern) {
 
 function metricTreeSelectorNodeClicked (node, e) {
   if (!node.leaf) {
-    node.toggle();
+    if (node.expanded) {
+      node.collapse();
+    } else {
+      node.loaded = false;
+      node.expand();
+    }
     return;
   }
 
@@ -944,7 +1008,7 @@ function graphAreaToggle(target, options) {
     var record = new GraphRecord({
       target: graphTargetString,
       params: myParams,
-      url: '/render?' + Ext.urlEncode(urlParams)
+      url: document.body.dataset.baseUrl + 'render?' + Ext.urlEncode(urlParams)
     });
     graphStore.add([record]);
     updateGraphRecords();
@@ -970,7 +1034,7 @@ function importGraphUrl(targetUrl, options) {
   if (graphTargetList.length == 0) {
     return;
   }
- 
+
   var graphTargetString = Ext.urlEncode({target: graphTargetList});
   var existingIndex = graphStore.findExact('target', graphTargetString);
 
@@ -987,7 +1051,7 @@ function importGraphUrl(targetUrl, options) {
     var record = new GraphRecord({
       target: graphTargetString,
       params: params,
-      url: '/render?' + Ext.urlEncode(urlParams)
+      url: document.body.dataset.baseUrl + 'render?' + Ext.urlEncode(urlParams)
       });
       graphStore.add([record]);
       updateGraphRecords();
@@ -1007,7 +1071,16 @@ function updateGraphRecords() {
     if (!params.uniq === undefined) {
         delete params["uniq"];
     }
-    item.set('url', '/render?' + Ext.urlEncode(params));
+
+    //Preload the image and set it into the UI once it is available.
+    item.set('loading','-loading');
+    var img = new Image();
+    img.onload = function() {
+      item.set('url',img.src);
+      item.set('loading','');
+    };
+    img.src = document.body.dataset.baseUrl + 'render?' + Ext.urlEncode(params);
+
     item.set('width', GraphSize.width);
     item.set('height', GraphSize.height);
     item.set('index', index);
@@ -1362,7 +1435,7 @@ function newFromSavedGraph() {
     expandable: true,
     allowDrag: false,
     loader: new Ext.tree.TreeLoader({
-      url: "../browser/usergraph/",
+      url: document.body.dataset.baseUrl + "browser/usergraph/",
       requestMethod: "GET",
       listeners: {beforeload: setParams}
     })
@@ -1371,7 +1444,7 @@ function newFromSavedGraph() {
   function handleSelects(selModel, nodes) {
     Ext.each(nodes, function (node, index) {
       if (!node.leaf) {
-	node.unselect();
+        node.unselect();
         node.toggle();
       }
     });
@@ -1434,6 +1507,71 @@ function newFromSavedGraph() {
   win.show();
 }
 
+function newFromMetric() {
+  function applyMetric() {
+    var inputMetric = Ext.getCmp('import-metric-field').getValue();
+    if (inputMetric == "") {
+      return;
+    }
+    var graphTargetString = Ext.urlEncode({target: inputMetric});
+
+    var myParams = {
+      target: [inputMetric]
+    };
+
+    var urlParams = {};
+    Ext.apply(urlParams, defaultGraphParams);
+    Ext.apply(urlParams, myParams);
+    Ext.apply(urlParams, GraphSize);
+
+    var record = new GraphRecord({
+      target: graphTargetString,
+      params: myParams,
+      url: '/render?' + Ext.urlEncode(urlParams)
+      });
+    graphStore.add([record]);
+    updateGraphRecords();
+    win.close();
+  }
+
+  var urlField = new Ext.form.TextField({
+    id: 'import-metric-field',
+    fieldLabel: "Metric",
+    region: 'center',
+    width: '100%',
+    listeners: {
+      specialkey: function (field, e) {
+                    if (e.getKey() == e.ENTER) {
+                      applyMetric();
+                    }
+                  },
+      afterrender: function (field) { field.focus(false, 100); }
+    }
+  });
+
+  var win = new Ext.Window({
+    title: "Import Graph From Metric",
+    width: 470,
+    height: 87,
+    layout: 'form',
+    resizable: true,
+    modal: true,
+    items: [urlField],
+    buttonAlign: 'center',
+    buttons: [
+      {
+        text: 'OK',
+        handler: applyMetric
+      }, {
+        text: 'Cancel',
+        handler: function () { win.close(); }
+      }
+    ]
+  });
+  win.show();
+}
+
+
 function editDefaultGraphParameters() {
   var editParams = Ext.apply({}, defaultGraphParams);
   removeUneditable(editParams);
@@ -1566,7 +1704,7 @@ function selectGraphSize() {
 function doShare() {
   if (dashboardName == null) {
     Ext.Ajax.request({
-      url: "/dashboard/create-temporary/",
+      url: document.body.dataset.baseUrl + "dashboard/create-temporary/",
       method: 'POST',
       params: {
         state: Ext.encode( getState() )
@@ -1870,7 +2008,7 @@ function graphClicked(graphView, graphIndex, element, evt) {
         }
         Ext.Ajax.request({
           method: 'GET',
-          url: '/s' + record.data.url,
+          url: document.body.dataset.baseUrl + 's' + record.data.url,
           callback: showUrl,
         });
       }
@@ -2000,7 +2138,7 @@ function breakoutGraph(record) {
   }
 
   Ext.Ajax.request({
-    url: '/metrics/expand/',
+    url: document.body.dataset.baseUrl + 'metrics/expand/',
     params: {
       groupByExpr: '1',
       leavesOnly: '1',
@@ -2071,7 +2209,7 @@ function mailGraph(record) {
          handler: function(){
            if(contactForm.getForm().isValid()){
              contactForm.getForm().submit({
-               url: '/dashboard/email',
+               url: document.body.dataset.baseUrl + 'dashboard/email',
                waitMsg: 'Processing Request',
                success: function (contactForm, response) {
          console.log(response.result);
@@ -2298,6 +2436,7 @@ function editDashboard() {
     var graphString = editor.getSession().getValue();
     var targets = JSON.parse(graphString);
     graphStore.removeAll();
+    var graphs = [];
     for (var i = 0; i < targets.length; i++) {
       var myParams = {};
       Ext.apply(myParams, targets[i]);
@@ -2305,13 +2444,14 @@ function editDashboard() {
       Ext.apply(urlParams, defaultGraphParams);
       Ext.apply(urlParams, GraphSize);
       Ext.apply(urlParams, myParams);
-      var record = new GraphRecord({
-        target: targets[i].target,
-        params: myParams,
-        url: '/render?' + Ext.urlEncode(urlParams)
-      });
-      graphStore.add([record]);
+      graphs.push([
+        Ext.urlEncode({target: targets[i].target}),
+        myParams,
+        document.body.dataset.baseUrl + 'render?' + Ext.urlEncode(urlParams)
+      ]);
     }
+    graphStore.loadData(graphs);
+    refreshGraphs();
     edit_dashboard_win.close();
   }
   function getInitialState() {
@@ -2353,9 +2493,69 @@ function saveDashboard() {
   );
 }
 
+function saveTemplate() {
+  var nameField = new Ext.form.TextField({
+    id: 'dashboard-save-template-name',
+    fieldLabel: "Template Name",
+    width: 240,
+    allowBlank: false,
+    align: 'center',
+    value: dashboardName ? dashboardName.split('/')[0]: '',
+  });
+
+  var keyField = new Ext.form.TextField({
+    id: 'dashboard-save-template-key',
+    fieldLabel: "String to replace",
+    width: 240,
+    allowBlank: false,
+    align: 'center',
+  });
+
+  var win;
+
+  function save() {
+    sendSaveTemplateRequest(nameField.getValue(), keyField.getValue());
+    win.close();
+  }
+
+  win = new Ext.Window({
+    title: "Save dashboard as a template",
+    width: 400,
+    height: 120,
+    resizable: false,
+    layout: 'form',
+    labelAlign: 'right',
+    labelWidth: 120,
+    items: [nameField,keyField],
+    buttonAlign: 'center',
+    buttons: [
+      {text: 'Ok', handler: save},
+      {text: 'Cancel', handler: function () { win.close(); } }
+    ]
+  });
+  win.show();
+}
+
+function sendSaveTemplateRequest(name, key) {
+  Ext.Ajax.request({
+    url: document.body.dataset.baseUrl + "dashboard/save_template/" + name + "/" + key,
+    method: 'POST',
+    params: {
+      state: Ext.encode( getState() )
+    },
+    success: function (response) {
+               var result = Ext.decode(response.responseText);
+               if (result.error) {
+                 Ext.Msg.alert("Error", "There was an error saving this dashboard as a template: " + result.error);
+               }
+             },
+    failure: failedAjaxCall
+  });
+}
+
 function sendSaveRequest(name) {
   Ext.Ajax.request({
-    url: "/dashboard/save/" + name,
+    url: document.body.dataset.baseUrl + "dashboard/save/" + name,
     method: 'POST',
     params: {
       state: Ext.encode( getState() )
@@ -2365,6 +2565,11 @@ function sendSaveRequest(name) {
                if (result.error) {
                  Ext.Msg.alert("Error", "There was an error saving this dashboard: " + result.error);
                }
+               if(newURL) {
+                 window.location = newURL;
+               } else {
+                 changeHash(name);
+               }
              },
     failure: failedAjaxCall
   });
@@ -2372,19 +2577,42 @@ function sendSaveRequest(name) {
 
 function sendLoadRequest(name) {
   Ext.Ajax.request({
-    url: "/dashboard/load/" + name,
+    url: document.body.dataset.baseUrl + "dashboard/load/" + name,
     success: function (response) {
                var result = Ext.decode(response.responseText);
                if (result.error) {
                  Ext.Msg.alert("Error Loading Dashboard", result.error);
                } else {
                  applyState(result.state);
+                 navBar.collapse(false);
                }
              },
     failure: failedAjaxCall
   });
 }
 
+function sendLoadTemplateRequest(name, value) {
+  urlparts = window.location.href.split('#')
+  if(urlparts[0].split('?')[1]) {
+    new_location = urlparts[0].split('?')[0] + '#'+name+'/'+value;
+    window.location.href = new_location;
+  } else {
+    Ext.Ajax.request({
+      url: document.body.dataset.baseUrl + "dashboard/load_template/" + name + "/" + value,
+      success: function (response) {
+               var result = Ext.decode(response.responseText);
+               if (result.error) {
+                 Ext.Msg.alert("Error Loading Template", result.error);
+               } else {
+                 applyState(result.state);
+                 navBar.collapse(false);
+               }
+             },
+      failure: failedAjaxCall
+    });
+  }
+}
+
 function getState() {
   var graphs = [];
   graphStore.each(
@@ -2425,6 +2653,24 @@ function applyState(state) {
   TimeRange.startTime = timeConfig.startTime;
   TimeRange.endDate = new Date(timeConfig.endDate);
   TimeRange.endTime = timeConfig.endTime;
+
+  if (queryString.from && queryString.until) {
+    // The URL contains a "from" and "until" parameters (format "YYYY-MM-DDThh:mm:ss") => use the timestamps as default absolute range of the dashboard
+    var from = new Date(queryString.from);
+    var until = new Date(queryString.until);
+
+    TimeRange.startDate = from;
+    TimeRange.startTime = from.format("H:m");
+    TimeRange.endDate = until;
+    TimeRange.endTime = until.format("H:m");
+    TimeRange.type = 'absolute';
+
+    state.timeConfig = TimeRange;
+
+    state.defaultGraphParams.from = from.format('H:i_Ymd');
+    state.defaultGraphParams.until = until.format('H:i_Ymd');
+  }
+
   updateTimeText();
 
 
@@ -2458,7 +2704,7 @@ function applyState(state) {
 
 function deleteDashboard(name) {
   Ext.Ajax.request({
-    url: "/dashboard/delete/" + name,
+    url: document.body.dataset.baseUrl + "dashboard/delete/" + name,
     success: function (response) {
       var result = Ext.decode(response.responseText);
       if (result.error) {
@@ -2471,11 +2717,26 @@ function deleteDashboard(name) {
   });
 }
 
+function deleteTemplate(name) {
+  Ext.Ajax.request({
+    url: document.body.dataset.baseUrl + "dashboard/delete_template/" + name,
+    success: function (response) {
+      var result = Ext.decode(response.responseText);
+      if (result.error) {
+        Ext.Msg.alert("Error", "Failed to delete template '" + name + "': " + result.error);
+      } else {
+        Ext.Msg.alert("Template Deleted", "The " + name + " template was deleted successfully.");
+      }
+    },
+    failure: failedAjaxCall
+  });
+}
+
 function setDashboardName(name) {
   dashboardName = name;
   var saveButton = Ext.getCmp('dashboard-save-button');
 
-  if (name == null) {
+  if (name == null || !hasPermission('change')) {
     dashboardURL = null;
     document.title = "untitled - Graphite Dashboard";
     navBar.setTitle("untitled");
@@ -2493,7 +2754,7 @@ function setDashboardName(name) {
     dashboardURL = urlparts.join('/');
 
     document.title = name + " - Graphite Dashboard";
-    window.location.hash = name;
+    changeHash(name);
     navBar.setTitle(name + " - (" + dashboardURL + ")");
     saveButton.setText('Save "' + name + '"');
     saveButton.enable();
@@ -2562,11 +2823,16 @@ function updateNavBar(region) {
   if (region == NAV_BAR_REGION) {
     return;
   }
-  cookieProvider.set('navbar-region', region);
-  Ext.Msg.alert('Cookie Updated', "You must refresh the page to update the nav bar's location.");
-  //TODO prompt the user to save their dashboard and refresh for them
 
+  cookieProvider.set('navbar-region', region);
   NAV_BAR_REGION = region;
+
+  if (graphStore.getCount() == 0) {
+    window.location.reload()
+  } else {
+    Ext.Msg.alert('Cookie Updated', "You must refresh the page to update the nav bar's location.");
+    //TODO prompt the user to save their dashboard and refresh for them
+  }
 }
 
 // Dashboard Finder
@@ -2575,15 +2841,25 @@ function showDashboardFinder() {
   var dashboardsList;
   var queryField;
   var dashboardsStore = new Ext.data.JsonStore({
-    url: "/dashboard/find/",
+    url: document.body.dataset.baseUrl + "dashboard/find/",
     method: 'GET',
     params: {query: "e"},
-    fields: ['name'],
+    fields: [{
+      name: 'name',
+      sortType: function(value) {
+	// Make sorting case-insensitive
+        return value.toLowerCase();
+      }
+    }],
     root: 'dashboards',
+    sortInfo: {
+      field: 'name',
+      direction: 'ASC'
+    },
     listeners: {
       beforeload: function (store) {
                     store.setBaseParam('query', queryField.getValue());
-                  }
+      }
     }
   });
 
@@ -2629,7 +2905,11 @@ function showDashboardFinder() {
                            Ext.getCmp('finder-delete-button').disable();
                          } else {
                            Ext.getCmp('finder-open-button').enable();
-                           Ext.getCmp('finder-delete-button').enable();
+                           if (hasPermission('delete')) {
+                             Ext.getCmp('finder-delete-button').enable();
+                           } else {
+                             Ext.getCmp('finder-delete-button').disable();
+                           }
                          }
                        },
 
@@ -2705,6 +2985,146 @@ function showDashboardFinder() {
   win.show();
 }
 
+// Template Finder
+function showTemplateFinder() {
+  var win;
+  var templatesList;
+  var queryField;
+  var valueField;
+  var templatesStore = new Ext.data.JsonStore({
+    url: document.body.dataset.baseUrl + "dashboard/find_template/",
+    method: 'GET',
+    params: {query: "e"},
+    fields: ['name'],
+    root: 'templates',
+    listeners: {
+      beforeload: function (store) {
+                    store.setBaseParam('query', queryField.getValue());
+                  }
+    }
+  });
+
+  function openSelected() {
+    var selected = templatesList.getSelectedRecords();
+    if (selected.length > 0) {
+      sendLoadTemplateRequest(selected[0].data.name, valueField.getValue());
+    }
+    win.close();
+  }
+
+  function deleteSelected() {
+    var selected = templatesList.getSelectedRecords();
+    if (selected.length > 0) {
+      var record = selected[0];
+      var name = record.data.name;
+
+      Ext.Msg.confirm(
+       "Delete Template",
+        "Are you sure you want to delete the " + name + " template?",
+        function (button) {
+          if (button == 'yes') {
+            deleteTemplate(name);
+            templatesStore.remove(record);
+            templatesList.refresh();
+          }
+        }
+      );
+    }
+  }
+
+  templatesList = new Ext.list.ListView({
+    columns: [
+      {header: 'Template', width: 1.0, dataIndex: 'name', sortable: false}
+    ],
+    columnSort: false,
+    emptyText: "No templates found",
+    hideHeaders: true,
+    listeners: {
+      selectionchange: function (listView, selections) {
+                         if (listView.getSelectedRecords().length == 0) {
+                           Ext.getCmp('finder-open-button').disable();
+                           Ext.getCmp('finder-delete-button').disable();
+                         } else {
+                           if (valueField.getValue()) {
+                             Ext.getCmp('finder-open-button').enable();
+                           }
+                           Ext.getCmp('finder-delete-button').enable();
+                         }
+                       },
+
+    },
+    overClass: '',
+    region: 'center',
+    reserveScrollOffset: true,
+    singleSelect: true,
+    store: templatesStore,
+    style: "background-color: white;"
+  });
+
+  var lastQuery = null;
+  var queryUpdateTask = new Ext.util.DelayedTask(
+    function () {
+      var currentQuery = queryField.getValue();
+      if (lastQuery != currentQuery) {
+        templatesStore.load();
+      }
+      lastQuery = currentQuery;
+    }
+  );
+
+  queryField = new Ext.form.TextField({
+    region: 'south',
+    emptyText: "filter template listing",
+    enableKeyEvents: true,
+    listeners: {
+      keyup: function (field, e) {
+                  if (e.getKey() == e.ENTER) {
+                    sendLoadRequest(field.getValue(), reset_params=true);
+                    win.close();
+                  } else {
+                    queryUpdateTask.delay(FINDER_QUERY_DELAY);
+                  }
+                }
+    }
+  });
+
+  valueField = new Ext.form.TextField({
+    region: 'north',
+    emptyText: "Value to use"
+  });
+
+  win = new Ext.Window({
+    title: "Template Finder",
+    width: 400,
+    height: 500,
+    layout: 'border',
+    modal: true,
+    items: [
+      valueField,
+      templatesList,
+      queryField,
+    ],
+    buttons: [
+      {
+        id: 'finder-open-button',
+        text: "Open",
+        disabled: true,
+        handler: openSelected
+      }, {
+        id: 'finder-delete-button',
+        text: "Delete",
+        disabled: true,
+        handler: deleteSelected
+      }, {
+        text: "Close",
+        handler: function () { win.close(); }
+      }
+    ]
+  });
+  templatesStore.load();
+  win.show();
+}
+
 /* Graph Options API (to reuse createOptionsMenu from composer_widgets.js) */
 function updateGraph() {
   refreshGraphs();
@@ -2764,11 +3184,7 @@ function applyFuncToEach(funcName, extraArg) {
         removeTargetFromSelectedGraph(target);
 
         if (extraArg) {
-          if (funcName == 'mostDeviant') { //SPECIAL CASE HACK
-            newTarget = funcName + '(' + extraArg + ',' + target + ')';
-          } else {
-            newTarget = funcName + '(' + target + ',' + extraArg + ')';
-          }
+          newTarget = funcName + '(' + target + ',' + extraArg + ')';
         } else {
           newTarget = funcName + '(' + target + ')';
         }
@@ -2844,7 +3260,9 @@ function removeOuterCall() { // blatantly repurposed from composer_widgets.js (d
     for (i = 0; i < argString.length; i++) {
       switch (argString.charAt(i)) {
         case '(': depth += 1; break;
+        case '{': depth += 1; break;
         case ')': depth -= 1; break;
+        case '}': depth -= 1; break;
         case ',':
           if (depth > 0) { continue; }
           if (depth < 0) { Ext.Msg.alert("Malformed target, cannot remove outer call."); return; }
@@ -2901,3 +3319,102 @@ function map(myArray, myFunc) {
   }
   return results;
 }
+
+function getLoginMenuItemText() {
+  if (isLoggedIn()) {
+    return 'Log Out From "' + userName + '"';
+  } else {
+    return 'Log In';
+  }
+}
+
+/* After login/logout, make any necessary adjustments to Dashboard menu items (text and/or disabled) */
+function postLoginMenuAdjust() {
+  Ext.getCmp("dashboard-login-button").setText(getLoginMenuItemText());
+  Ext.getCmp("dashboard-save-button").setDisabled(dashboardName == null || !hasPermission('change'));
+  Ext.getCmp("dashboard-save-as-button").setDisabled(!hasPermission('change'));
+}
+
+function showLoginForm() {
+  var login = new Ext.FormPanel({
+    labelWidth: 80,
+    frame: true,
+    title: 'Please Login',
+    defaultType: 'textfield',
+    monitorValid: true,
+
+    items: [{
+        fieldLabel: 'Username',
+        name: 'username',
+        allowBlank: false,
+        listeners: {
+          afterrender: function(field) { field.focus(false, 100); }
+        }
+      },{
+        fieldLabel: 'Password',
+        name: 'password',
+        inputType: 'password',
+        allowBlank: false
+      }
+    ],
+    buttons: [
+      {text: 'Login', formBind: true, handler: doLogin},
+      {text: 'Cancel', handler: function () { win.close(); } }
+    ]
+  });
+
+  function doLogin() {
+    login.getForm().submit({
+      method: 'POST',
+      url: document.body.dataset.baseUrl + 'dashboard/login',
+      waitMsg: 'Authenticating...',
+      success: function(form, action) {
+        userName = form.findField('username').getValue();
+        permissions = action.result.permissions;
+        postLoginMenuAdjust();
+        win.close();
+      },
+      failure: function(form, action) {
+        if (action.failureType == 'server') {
+          var obj = Ext.util.JSON.decode(action.response.responseText);
+          Ext.Msg.alert('Login Failed!', obj.errors.reason);
+        } else {
+          Ext.Msg.alert('Warning!', 'Authentication server is unreachable : ' + action.response.responseText);
+        }
+        login.getForm().reset();
+      }
+    });
+  }
+
+  var win = new Ext.Window({
+    layout: 'fit',
+    width: 300,
+    height: 150,
+    closable: false,
+    resizable: false,
+    plain: true,
+    border: false,
+    items: [login]
+  });
+  win.show();
+}
+
+function logout() {
+  Ext.Ajax.request({
+    url: document.body.dataset.baseUrl + 'dashboard/logout',
+    method: 'POST',
+    success: function() {
+      userName = null;
+      permissions = permissionsUnauthenticated;
+      postLoginMenuAdjust();
+    },
+    failure: function() {
+      // Probably because they no longer have a valid session - assume they're now logged out
+      userName = null;
+      permissions = permissionsUnauthenticated;
+      postLoginMenuAdjust();
+    }
+  });
+}
+
+
