@@ -6,11 +6,13 @@ from graphite.url_shortener.baseconv import base62
 from graphite.url_shortener.models import Link
 import re
 
+
 def follow(request, link_id):
     """Follow existing links"""
     key = base62.to_decimal(link_id)
     link = get_object_or_404(Link, pk=key)
-    return HttpResponsePermanentRedirect('/' + link.url)
+    return HttpResponsePermanentRedirect(reverse('browser') + link.url)
+
 
 def shorten(request, path):
     if request.META.get('QUERY_STRING', None):
@@ -20,5 +22,5 @@ def shorten(request, path):
 
     link, created = Link.objects.get_or_create(url=path)
     link_id = base62.from_decimal(link.id)
-    url = reverse('graphite.url_shortener.views.follow', kwargs={'link_id': link_id})
-    return HttpResponse(url, content_type='text/plain')
\ No newline at end of file
+    url = reverse('follow', kwargs={'link_id': link_id})
+    return HttpResponse(url, content_type='text/plain')
