@@ -12,11 +12,9 @@ You may obtain a copy of the License at
    See the License for the specific language governing permissions and
    limitations under the License."""
 
-try:
-    from django.conf.urls.defaults import *
-except ImportError:
-    from django.conf.urls import *
+from django.conf.urls import url
+from . import views
 
-urlpatterns = patterns('graphite.version.views',
-  ('', 'index'),
-)
+urlpatterns = [
+    url('', views.index, name='version_index'),
+]
